#pragma once

#include <FlashTool/FlashTool.h>
#include "Grabber.h"

class FlvDataCollectorObserver;

//=====================================================================================//
//                               class FlvDataCollector                                //
//=====================================================================================//
class FlvDataCollector : public GrabberObserver
{
private:

	//=====================================================================================//
	//                                    struct Buffer                                    //
	//=====================================================================================//
	struct Buffer
	{
		unsigned int m_timestamp;
		std::vector<char> m_data;

		Buffer(const char* data,unsigned int size,unsigned int timestamp);
	};
	//=====================================================================================//

	typedef void (*ftFlvStreamPutData)(HFTFLVSTREAM hFlvStream, LPCVOID lpData, DWORD nLen,DWORD nTimeStamp);
	typedef std::vector<Buffer> buffers_t;

	HFTSCREENVIDEOENCODER m_hScreenVideoEncoder;
	FlvDataCollectorObserver* m_observer;
	HFTFLVSTREAM m_hFlvStream;

	unsigned int m_videoTimestamp;
	unsigned int m_audioTimestamp;
	unsigned int m_frameDuration;
	buffers_t m_audio;

public:
	FlvDataCollector(HFTFLVSTREAM hFlvStream,FlvDataCollectorObserver* observer,
					 unsigned int image_width,unsigned int image_height,
					 unsigned int frame_rate);
	~FlvDataCollector();
public:
	///	flush data
	void flush();

private:
	///	process video data callback
	void onVideoData(const char* data,unsigned int size,unsigned int timestamp);
	///	process audio data callback
	void onAudioData(const char* data,unsigned int size,unsigned int timestamp);

private:
	///	flush all stored data buffers
	void flushStoredBuffers(buffers_t& buffers,ftFlvStreamPutData func);
	///	flush stored data buffers for current timestamp
	void flushStoredBuffers(buffers_t& buffers,ftFlvStreamPutData func,unsigned int timestamp);

};

//=====================================================================================//
//                           class FlvDataCollectorObserver                            //
//=====================================================================================//
class FlvDataCollectorObserver
{
public:
	virtual ~FlvDataCollectorObserver() {}
	///	process progress of converting
	virtual void onProgress(unsigned int timestamp) = 0;

};
