#pragma once

#include "AudioGrabber.h"
#include "VideoGrabber.h"

class GrabberObserver;

//=====================================================================================//
//                                    class Grabber                                    //
//=====================================================================================//
class Grabber : public VideoGrabberObserver,
				public AudioGrabberObserver
{
public:

	//=====================================================================================//
	//                                    enum Options                                     //
	//=====================================================================================//
	enum Options
	{
		Video	=	1<<0,
		Audio	=	1<<1
	};
	//=====================================================================================//

private:

	typedef GrabberObserver Observer;

	ATL::CComPtr<ISampleGrabber> m_sampleVideoGrabber;
	ATL::CComPtr<ISampleGrabber> m_sampleAudioGrabber;
	ATL::CComPtr<IBaseFilter> m_nullVideoFilter;
	ATL::CComPtr<IBaseFilter> m_nullSoundFilter;
    ATL::CComPtr<IGraphBuilder> m_graphBuilder;
	ATL::CComPtr<IBaseFilter> m_sourceFilter;
	ATL::CComPtr<IBaseFilter> m_aviSplitter;

	ATL::CComPtr<IMediaControl> m_mediaControl;
	ATL::CComPtr<IMediaSeeking> m_mediaSeeking;
	ATL::CComPtr<IMediaEventEx> m_mediaEvent;

	ATL::CComPtr<ISampleGrabberCB> m_videoGrabberCB;
	ATL::CComPtr<ISampleGrabberCB> m_audioGrabberCB;

	CRITICAL_SECTION m_criticalSection;

	Observer* m_observer;

public:
	Grabber(const std::string& file_name,unsigned int options = Options::Video|Options::Audio);
	~Grabber();
public:
	///	run process
	void run(GrabberObserver* observer);

private:
	///	setup general settings for graph builder
	void setupGeneralSettings(const std::string& file_name);
	///	setup video grabber settings
	void setupVideoGrabber();
	///	setup audio grabber settings
	void setupAudioGrabber();

private:
	///	process video data callback
	void onVideoData(const char* data,unsigned int size,unsigned int timestamp);
	///	process audio data callback
	void onAudioData(const char* data,unsigned int size,unsigned int timestamp);

};

//=====================================================================================//
//                                class GrabberObserver                                //
//=====================================================================================//
class GrabberObserver
{
public:
	virtual ~GrabberObserver() {}
	///	process video data callback
	virtual void onVideoData(const char* data,unsigned int size,unsigned int timestamp) = 0;
	///	process audio data callback
	virtual void onAudioData(const char* data,unsigned int size,unsigned int timestamp) = 0;

};
